library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity parkhaus is
	Port ( 	reset	: in  STD_LOGIC;
		clk	: in  STD_LOGIC;
		L1	: in  STD_LOGIC;
		L2	: in  STD_LOGIC;
		cnt	: out  STD_LOGIC_VECTOR (2 downto 0);
		err	: out  STD_LOGIC);
end parkhaus;


architecture Behavioral of parkhaus is
	type states is (	def,	-- Default State
				Zm,	-- Z--?
				Zmm,	-- Z--
				Zp,	-- Z++?
				Zpp,	-- Z++
				wt);	-- wait
	
	-- Speicherung des aktuellen Zustands
	signal state : states;
	-- Speicherung des folgenden Zustands
	signal state_next : states;

	-- Register f. Zählerstand
	signal cntbuf : STD_LOGIC_VECTOR(2 downto 0) :="101";
	-- Register f. Fehlerfall
	signal errbuf : STD_LOGIC :='0';
begin

state_transitions: process(state,L1,L2)
begin
	-- default state
	state_next <= def;

	case state is
		when def 	=>
			-- default Zuweisung
			state_next <= def;
			
			if 	L1='1' and L2='0' then
	
				state_next <= Zp;
	
			elsif	L1='0' and L2='1' then
	
				state_next <= Zm;
			end if;
	
		when Zm 	=>
			-- default Zuweisung
			state_next <= Zm;
	
			if 	L1='0' and L2='0' then
	
					state_next <= def;
	
			elsif 	L1='1' and L2='0' then
	
					state_next <= Zmm;
			end if;
	
		when Zmm 	=>
			-- default Zuweisung
			state_next <= wt;
	
		when Zp		=>
			-- default Zuweisung
			state_next <= Zp;
	
			if 	L1='0' and L2='0' then
	
				state_next <= def;
	
			elsif 	L1='0' and L2='1' then
	
				state_next <= Zpp;
			end if;
	
		when Zpp	=>
			-- default Zuweisung
			state_next <= wt;
	
		when wt		=>
			-- default Zuweisung
			state_next <= wt;
	
			if	 L1='0' and L2='0' then
	
				state_next <= def;
			end if;
	
	end case;

end process state_transitions;

state_logic: process(clk)
begin
	if clk'event and clk='1' then
	-- Hauptschleife
		-- positiver Reset
		if reset='1' then
			state <= def;		-- erreichen des Defaultstates
			cntbuf <= "101";	-- den Zähler auf fünf setzen
			errbuf <= '0';		-- den Fehlerfall entfernen
		else

			-- state-Register-Transition
			state <= state_next;

			if state=Zpp then

				if cntbuf="101" then
					errbuf <= '1';
				elsif cntbuf="000" and errbuf='1' then
					errbuf <= '0';
				else
					cntbuf <= cntbuf + 1;
				end if;
			
			elsif state=Zmm then
			
				if cntbuf="000" then
					errbuf <= '1';
				elsif cntbuf="101" and errbuf='1' then
					errbuf <= '0';
				else
					cntbuf <= cntbuf - 1;
				end if;
			
			end if;
		end if;
	end if;

end process state_logic;


	-- Simulation
--	cnt <= cntbuf;
--	err <= errbuf;
	
	-- Synthese auf Test-Board (LEDs sind Low-Aktiv)
	cnt <= not cntbuf;
	err <= not errbuf;

end Behavioral;
